"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = y[op[0] & 2 ? "return" : op[0] ? "throw" : "next"]) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [0, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
Object.defineProperty(exports, "__esModule", { value: true });
var yaml_1 = require("./yaml");
var fs = require("fs-extra");
var lodash_1 = require("lodash");
var yamlParser = require("yaml-ast-parser");
var StageNotFound_1 = require("../errors/StageNotFound");
var dotenv = require("dotenv");
var path = require("path");
var jwt = require("jsonwebtoken");
var GraphcoolDefinitionClass = (function () {
    function GraphcoolDefinitionClass(out, config) {
        var _this = this;
        this.resolveStageAliases = function (stages) {
            return lodash_1.mapValues(stages, function (target) { return _this.resolveStage(target, stages); });
        };
        this.resolveStage = function (stage, stages) {
            return stages[stage] ? _this.resolveStage(stages[stage], stages) : stage;
        };
        this.out = out;
        this.config = config;
        this.secrets = null;
    }
    GraphcoolDefinitionClass.prototype.load = function (env, args, envPath) {
        return __awaiter(this, void 0, void 0, function () {
            var _a;
            return __generator(this, function (_b) {
                switch (_b.label) {
                    case 0:
                        if (!this.config.definitionPath) return [3 /*break*/, 2];
                        dotenv.config({ path: envPath });
                        _a = this;
                        return [4 /*yield*/, yaml_1.readDefinition(this.config.definitionPath, this.out, args)];
                    case 1:
                        _a.definition = _b.sent();
                        this.definitionString = fs.readFileSync(this.config.definitionPath, 'utf-8');
                        this.rawStages = this.definition.stages;
                        this.definition.stages = this.resolveStageAliases(this.definition.stages);
                        this.ensureOfClusters(this.definition, env);
                        this.typesString = this.getTypesString(this.definition);
                        this.secrets = this.definition.secret
                            ? this.definition.secret.replace(/\s/g, '').split(',')
                            : null;
                        if (this.secrets === null && !this.definition.disableAuth) {
                            throw new Error('Please either provide a secret in your graphcool.yml or disableAuth: true');
                        }
                        return [3 /*break*/, 3];
                    case 2: throw new Error("Please create a graphcool.yml");
                    case 3: return [2 /*return*/];
                }
            });
        });
    };
    GraphcoolDefinitionClass.prototype.getToken = function (serviceName, stageName) {
        if (this.secrets) {
            var data = {
                data: {
                    service: serviceName + "@" + stageName,
                    roles: ['admin'],
                },
            };
            return jwt.sign(data, this.secrets[0], {
                expiresIn: '7d',
            });
        }
        return undefined;
    };
    GraphcoolDefinitionClass.prototype.getStage = function (name, throws) {
        if (throws === void 0) { throws = false; }
        var stage = this.definition &&
            (this.definition.stages[name] || this.definition.stages.default);
        if (!throws) {
            return stage;
        }
        if (!stage) {
            throw new StageNotFound_1.StageNotFound(name);
        }
        return stage;
    };
    GraphcoolDefinitionClass.prototype.setStage = function (name, clusterName) {
        this.definitionString = this.insertToDefinition(this.definitionString, 'stages', "\n  " + name + ": " + clusterName);
    };
    GraphcoolDefinitionClass.prototype.insertToDefinition = function (file, key, insertion) {
        var obj = yamlParser.safeLoad(file);
        var mapping = obj.mappings.find(function (m) { return m.key.value === key; });
        if (mapping) {
            var end = mapping.endPosition;
            var newFile = file.slice(0, end) + insertion + file.slice(end);
            var valueStart = mapping.value.startPosition;
            var valueEnd = mapping.value.endPosition;
            if (mapping.value && valueEnd - valueStart < 4) {
                return newFile.slice(0, valueStart) + newFile.slice(valueEnd);
            }
            return newFile;
        }
        else {
            return file + ("\n" + key + ": ") + insertion;
        }
    };
    GraphcoolDefinitionClass.prototype.save = function () {
        fs.writeFileSync(this.config.definitionPath, this.definitionString);
    };
    GraphcoolDefinitionClass.prototype.getTypesString = function (definition) {
        var _this = this;
        var typesPaths = Array.isArray(definition.datamodel)
            ? definition.datamodel
            : [definition.datamodel];
        var errors = [];
        var allTypes = '';
        typesPaths.forEach(function (unresolvedTypesPath) {
            var typesPath = path.join(_this.config.definitionDir, unresolvedTypesPath);
            if (fs.existsSync(typesPath)) {
                var types = fs.readFileSync(typesPath, 'utf-8');
                allTypes += types + '\n';
            }
            else {
                throw new Error("The types definition file \"" + typesPath + "\" could not be found.");
            }
        });
        return allTypes;
    };
    GraphcoolDefinitionClass.prototype.ensureOfClusters = function (definition, env) {
        Object.keys(definition.stages).forEach(function (stageName) {
            var referredCluster = definition.stages[stageName];
            if (!env.clusters.find(function (c) { return c.name === referredCluster; })) {
                throw new Error("Could not find cluster '" + referredCluster + "', which is used in stage '" + stageName + "'.");
            }
        });
    };
    Object.defineProperty(GraphcoolDefinitionClass.prototype, "default", {
        get: function () {
            if (this.definition &&
                this.definition.stages &&
                this.definition.stages.default) {
                return this.definition.stages.default;
            }
            return null;
        },
        enumerable: true,
        configurable: true
    });
    return GraphcoolDefinitionClass;
}());
exports.GraphcoolDefinitionClass = GraphcoolDefinitionClass;
//# sourceMappingURL=GraphcoolDefinition.js.map